/**
 * @ag-grid-community/core - Advanced Data Grid / Data Table supporting Javascript / Typescript / React / Angular / Vue
 * @version v27.3.0-beta.0
 * @link https://www.ag-grid.com/
 * @license MIT
 */
"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
const eventKeys_1 = require("../../../eventKeys");
const dom_1 = require("../../../utils/dom");
const context_1 = require("../../../context/context");
const componentAnnotations_1 = require("../../../widgets/componentAnnotations");
const component_1 = require("../../../widgets/component");
const icon_1 = require("../../../utils/icon");
class SortIndicatorComp extends component_1.Component {
    constructor() {
        super();
        this.setTemplate(SortIndicatorComp.TEMPLATE);
    }
    setupSort(column) {
        this.column = column;
        this.setupMultiSortIndicator();
        const canSort = !!this.column.getColDef().sortable;
        if (!canSort) {
            return;
        }
        this.addInIcon('sortAscending', this.eSortAsc, column);
        this.addInIcon('sortDescending', this.eSortDesc, column);
        this.addInIcon('sortUnSort', this.eSortNone, column);
        // Watch global events, as row group columns can effect their display column.
        this.addManagedListener(this.eventService, eventKeys_1.Events.EVENT_SORT_CHANGED, () => this.onSortChanged());
        // when grouping changes so can sort indexes and icons
        this.addManagedListener(this.eventService, eventKeys_1.Events.EVENT_COLUMN_ROW_GROUP_CHANGED, () => this.onSortChanged());
        this.onSortChanged();
    }
    addInIcon(iconName, eParent, column) {
        if (eParent == null) {
            return;
        }
        const eIcon = icon_1.createIconNoSpan(iconName, this.gridOptionsWrapper, column);
        if (eIcon) {
            eParent.appendChild(eIcon);
        }
    }
    onSortChanged() {
        this.updateIcons();
        this.updateSortOrder();
    }
    updateIcons() {
        const isMixedSort = this.isColumnMixedSorting();
        const isAscending = !isMixedSort && this.column.isSortAscending();
        const isDescending = !isMixedSort && this.column.isSortDescending();
        const isNone = !isMixedSort && this.column.isSortNone();
        if (this.eSortAsc) {
            this.eSortAsc.classList.toggle('ag-hidden', !isAscending);
        }
        if (this.eSortDesc) {
            this.eSortDesc.classList.toggle('ag-hidden', !isDescending);
        }
        if (this.eSortNone) {
            const alwaysHideNoSort = !this.column.getColDef().unSortIcon && !this.gridOptionsWrapper.isUnSortIcon();
            this.eSortNone.classList.toggle('ag-hidden', (alwaysHideNoSort || !isNone));
        }
    }
    setupMultiSortIndicator() {
        this.addInIcon('sortUnSort', this.eSortMixed, this.column);
        // We only enable the mixed indicator for CSRM, as we know we always sort groups first.
        const isColumnShowingRowGroup = this.column.getColDef().showRowGroup;
        const isClientSideRowModel = this.gridOptionsWrapper.isRowModelDefault();
        if (isClientSideRowModel && isColumnShowingRowGroup) {
            // Watch global events, as row group columns can effect their display column.
            this.addManagedListener(this.eventService, eventKeys_1.Events.EVENT_SORT_CHANGED, () => this.updateMultiSortIndicator());
            // when grouping changes so can sort indexes and icons
            this.addManagedListener(this.eventService, eventKeys_1.Events.EVENT_COLUMN_ROW_GROUP_CHANGED, () => this.updateMultiSortIndicator());
            this.updateMultiSortIndicator();
        }
    }
    isColumnMixedSorting() {
        var _a;
        const isClientSideRowModel = this.gridOptionsWrapper.isRowModelDefault();
        if (!isClientSideRowModel) {
            return false;
        }
        if (!this.column.getColDef().showRowGroup) {
            return false;
        }
        const sourceColumns = this.columnModel.getSourceColumnsForGroupColumn(this.column);
        // this == is intentional, as it allows null and undefined to match, which are both unsorted states
        const sortDirectionsMatch = (_a = sourceColumns) === null || _a === void 0 ? void 0 : _a.every(sourceCol => this.column.getSort() == sourceCol.getSort());
        const isMultiSorting = !sortDirectionsMatch;
        return isMultiSorting;
    }
    updateMultiSortIndicator() {
        if (this.eSortMixed) {
            this.eSortMixed.classList.toggle('ag-hidden', !this.isColumnMixedSorting());
        }
    }
    // we listen here for global sort events, NOT column sort events, as we want to do this
    // when sorting has been set on all column (if we listened just for our col (where we
    // set the asc / desc icons) then it's possible other cols are yet to get their sorting state.
    updateSortOrder() {
        if (!this.eSortOrder) {
            return;
        }
        const allColumnsWithSorting = this.sortController.getColumnsWithSortingOrdered();
        const notRowGroupColumnsWithSorting = allColumnsWithSorting.filter(sortedCol => !this.columnModel.getGroupDisplayColumnForGroup(sortedCol.getId()));
        const isClientSideRowModel = this.gridOptionsWrapper.isRowModelDefault();
        const indexableColumns = isClientSideRowModel ? notRowGroupColumnsWithSorting : allColumnsWithSorting;
        const colIsNotRowGroup = !this.columnModel.getGroupDisplayColumnForGroup(this.column.getId());
        const indexThisCol = indexableColumns.indexOf(this.column);
        const moreThanOneColSorting = indexableColumns.length > 1;
        const showIndex = this.column.isSorting() && moreThanOneColSorting && (!isClientSideRowModel || colIsNotRowGroup);
        dom_1.setDisplayed(this.eSortOrder, showIndex);
        if (indexThisCol >= 0) {
            this.eSortOrder.innerHTML = (indexThisCol + 1).toString();
        }
        else {
            dom_1.clearElement(this.eSortOrder);
        }
    }
}
SortIndicatorComp.TEMPLATE = `<span class="ag-sort-indicator-container">
            <span ref="eSortOrder" class="ag-sort-indicator-icon ag-sort-order ag-hidden" aria-hidden="true"></span>
            <span ref="eSortAsc" class="ag-sort-indicator-icon ag-sort-ascending-icon ag-hidden" aria-hidden="true"></span>
            <span ref="eSortDesc" class="ag-sort-indicator-icon ag-sort-descending-icon ag-hidden" aria-hidden="true"></span>
            <span ref="eSortMixed" class="ag-sort-indicator-icon ag-sort-mixed-icon ag-hidden" aria-hidden="true"></span>
            <span ref="eSortNone" class="ag-sort-indicator-icon ag-sort-none-icon ag-hidden" aria-hidden="true"></span>
        </span>`;
__decorate([
    componentAnnotations_1.RefSelector('eSortOrder')
], SortIndicatorComp.prototype, "eSortOrder", void 0);
__decorate([
    componentAnnotations_1.RefSelector('eSortAsc')
], SortIndicatorComp.prototype, "eSortAsc", void 0);
__decorate([
    componentAnnotations_1.RefSelector('eSortDesc')
], SortIndicatorComp.prototype, "eSortDesc", void 0);
__decorate([
    componentAnnotations_1.RefSelector('eSortMixed')
], SortIndicatorComp.prototype, "eSortMixed", void 0);
__decorate([
    componentAnnotations_1.RefSelector('eSortNone')
], SortIndicatorComp.prototype, "eSortNone", void 0);
__decorate([
    context_1.Autowired('columnModel')
], SortIndicatorComp.prototype, "columnModel", void 0);
__decorate([
    context_1.Autowired('sortController')
], SortIndicatorComp.prototype, "sortController", void 0);
exports.SortIndicatorComp = SortIndicatorComp;

//# sourceMappingURL=sortIndicatorComp.js.map
